import { defineStore } from "pinia";
import type {
  InspectionPlanViewModel,
  CreateInspectionPlanViewModel,
  UpdateInspectionPlanViewModel,
} from "@/viewmodels/admin/unit/inspection/inspectionPlan.models";
import { http } from "@/serverCom";
import type { AxiosResponse } from "axios";

export const useInspectionPlanStore = defineStore("inspectionPlan", {
  state: () => {
    return {
      inspectionPlans: [] as Array<InspectionPlanViewModel & { tab_pos: number }>,
      totalCount: 0 as number,
      loading: "loading" as "loading" | "fetched" | "failed",
      activeInspectionPlan: null as string | null,
      activeInspectionPlanObj: null as InspectionPlanViewModel | null,
      loadingActive: "loading" as "loading" | "fetched" | "failed",
    };
  },
  actions: {
    fetchInspectionPlans(offset = 0, count = 25, search = "", clear = false) {
      if (clear) this.inspectionPlans = [];
      this.loading = "loading";
      http
        .get(`/admin/inspectionPlan?offset=${offset}&count=${count}${search != "" ? "&search=" + search : ""}`)
        .then((result) => {
          this.totalCount = result.data.total;
          result.data.inspectionPlans
            .filter((elem: InspectionPlanViewModel) => this.inspectionPlans.findIndex((m) => m.id == elem.id) == -1)
            .map((elem: InspectionPlanViewModel, index: number): InspectionPlanViewModel & { tab_pos: number } => {
              return {
                ...elem,
                tab_pos: index + offset,
              };
            })
            .forEach((elem: InspectionPlanViewModel & { tab_pos: number }) => {
              this.inspectionPlans.push(elem);
            });
          this.loading = "fetched";
        })
        .catch((err) => {
          this.loading = "failed";
        });
    },
    async getAllInspectionPlans(): Promise<AxiosResponse<any, any>> {
      return await http.get(`/admin/inspectionPlan?noLimit=true`).then((res) => {
        return { ...res, data: res.data.inspectionPlans };
      });
    },
    async searchInspectionPlans(search: string): Promise<AxiosResponse<any, any>> {
      return await http.get(`/admin/inspectionPlan?search=${search}&noLimit=true`).then((res) => {
        return { ...res, data: res.data.inspectionPlans };
      });
    },
    fetchInspectionPlanByActiveId() {
      this.loadingActive = "loading";
      http
        .get(`/admin/inspectionPlan/${this.activeInspectionPlan}`)
        .then((res) => {
          this.activeInspectionPlanObj = res.data;
          this.loadingActive = "fetched";
        })
        .catch((err) => {
          this.loadingActive = "failed";
        });
    },
    fetchInspectionPlanById(id: string) {
      return http.get(`/admin/inspectionPlan/${id}`);
    },
    async createInspectionPlan(inspectionPlan: CreateInspectionPlanViewModel): Promise<AxiosResponse<any, any>> {
      const result = await http.post(`/admin/inspectionPlan`, {
        title: inspectionPlan.title,
        inspectionInterval: inspectionPlan.inspectionInterval,
        remindTime: inspectionPlan.remindTime,
        relatedId: inspectionPlan.relatedId,
        assigned: inspectionPlan.assigned,
      });
      this.fetchInspectionPlans();
      return result;
    },
    async updateActiveInspectionPlan(inspectionPlan: UpdateInspectionPlanViewModel): Promise<AxiosResponse<any, any>> {
      const result = await http.patch(`/admin/inspectionPlan/${inspectionPlan.id}`, {
        title: inspectionPlan.title,
        inspectionInterval: inspectionPlan.inspectionInterval,
        remindTime: inspectionPlan.remindTime,
      });
      this.fetchInspectionPlans();
      return result;
    },
    async deleteInspectionPlan(inspectionPlan: number): Promise<AxiosResponse<any, any>> {
      const result = await http.delete(`/admin/inspectionPlan/${inspectionPlan}`);
      this.fetchInspectionPlans();
      return result;
    },
  },
});
