import type { InspectionPointEnum } from "@/enums/inspectionEnum";
import type { EquipmentViewModel } from "../equipment/equipment.models";
import type { VehicleViewModel } from "../vehicle/vehicle.models";
import type { EquipmentTypeViewModel } from "../equipment/equipmentType.models";
import type { VehicleTypeViewModel } from "../vehicle/vehicleType.models";
import type { WearableTypeViewModel } from "../wearable/wearableType.models";

export type PlanTimeDefinition = `${number}-${"d" | "m" | "y"}` | `${number}/${number | "*"}`;

export type InspectionPlanViewModel = {
  id: string;
  title: string;
  inspectionInterval: PlanTimeDefinition;
  remindTime: PlanTimeDefinition;
  version: number;
  created: Date;
  inspectionPoints: InspectionPointViewModel[];
  relatedId: string;
} & (
  | {
      assigned: "equipment";
      related: EquipmentTypeViewModel;
    }
  | {
      assigned: "vehicle";
      related: VehicleTypeViewModel;
    }
  | {
      assigned: "wearable";
      related: WearableTypeViewModel;
    }
);

export interface InspectionVersionedPlanViewModel {
  id: string;
  version: number;
  created: Date;
  inspectionPoints: InspectionPointViewModel[];
}

export interface InspectionPointViewModel {
  id: string;
  title: string;
  description: string;
  type: InspectionPointEnum;
  min?: number;
  max?: number;
  sort: number;
}

export interface CreateInspectionPlanViewModel {
  title: string;
  inspectionInterval: PlanTimeDefinition;
  remindTime: PlanTimeDefinition;
  relatedId: string;
  assigned: "vehicle" | "equipment" | "wearable";
}

export interface UpdateInspectionPlanViewModel {
  id: string;
  title: string;
  inspectionInterval: PlanTimeDefinition;
  remindTime?: PlanTimeDefinition;
}
